# Copyright 2022 Le Filament (https://le-filament.com)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html)

import json

from dateutil.relativedelta import relativedelta

from odoo import _, api, fields, models
from odoo.exceptions import UserError


class ResPartner(models.Model):
    _inherit = "res.partner"

    # ------------------------------------------------------
    # Fields declaration
    # ------------------------------------------------------
    contact_type = fields.Selection(
        [
            ("provider", "Prestataire"),
            ("project", "Projet"),
        ],
        string="Type Alter'Incub",
    )
    parent_id_domain = fields.Char(compute="_compute_parent_id_domain")
    is_incubated = fields.Boolean(
        string="Est porteur de projet",
        compute="_compute_is_incubated",
        store=True,
        default=False,
    )
    is_provider = fields.Boolean(
        string="Est intervenant",
        compute="_compute_is_provider",
        store=True,
        default=False,
    )
    # Provider fields
    is_approved = fields.Boolean("Référencé", default=False, tracking=True)
    is_vat = fields.Boolean("Exonération de TVA", default=False, tracking=True)
    lot_ids = fields.Many2many(
        comodel_name="alterincub.lot",
        relation="res_partner_lot_rel",
        column1="partner_id",
        column2="lot_id",
        string="Lots",
    )
    price_individual_ids = fields.One2many(
        comodel_name="alterincub.price.list",
        inverse_name="partner_id",
        domain=[("type_lot", "=", "ae")],
        string="Accompagnement Individualisé",
    )
    price_collective_ids = fields.One2many(
        comodel_name="alterincub.price.list",
        inverse_name="partner_id",
        domain=[("type_lot", "=", "jc")],
        string="Journées Collectives",
    )
    # Provider contact fields
    is_admin_contact = fields.Boolean(
        string="Contact Administratif", default=False, tracking=True
    )
    evaluation = fields.Selection(
        [
            ("0", "0"),
            ("1", "1"),
            ("2", "2"),
            ("3", "4"),
            ("4", "4"),
            ("5", "5"),
        ],
        "Évaluation",
        tracking=True,
    )
    provider_specificity = fields.Text("Spécificités")

    # Projects fields
    original_name = fields.Char(
        string="Nom initial",
        compute="_compute_original_name",
        store=True,
    )
    class_id = fields.Many2one(
        comodel_name="alterincub.class", string="Promotion", tracking=True
    )
    summary = fields.Text("Résumé du projet")
    department_id = fields.Many2one(
        comodel_name="alterincub.department",
        string="Territoire d’implantation",
        ondelete="restrict",
    )
    area_id = fields.Many2one(
        comodel_name="alterincub.area",
        related="department_id.area_id",
        string="Territoire",
        store="True",
    )
    sector_id = fields.Many2one(
        comodel_name="alterincub.sector",
        string="Secteur d’activité",
        ondelete="restrict",
        tracking=True,
    )
    sector_2_id = fields.Many2one(
        comodel_name="alterincub.sector",
        string="Secteur d’activité 2",
        ondelete="restrict",
        tracking=True,
    )
    consultant_id = fields.Many2one(
        comodel_name="res.users",
        string="Consultant.e Alter’Incub",
        domain=[("share", "=", False)],
        ondelete="restrict",
        tracking=True,
        default=lambda self: self.env.user,
    )
    program_id = fields.Many2one(
        comodel_name="alterincub.program",
        string="Programme",
        ondelete="restrict",
        tracking=True,
    )

    previous_program_ids = fields.Many2many(
        comodel_name="alterincub.program",
        string="Programmes précédents",
    )

    parent_program_id = fields.Many2one(
        related="parent_id.program_id",
        string="Programme Parent",
        store=True,
    )
    start_date = fields.Date(
        string="Date de début",
        tracking=True,
    )
    stop_date = fields.Date(
        string="Date de fin d’accompagnement",
        tracking=True,
    )
    end_reason_id = fields.Many2one(
        comodel_name="alterincub.end.reason",
        string="Motif de sortie prématurée",
        ondelete="restrict",
        tracking=True,
    )
    end_reason_comment = fields.Text(string="Détails de sortie prématurée")
    project_state = fields.Selection(
        [
            ("0_pre", "Pré-incubation"),
            ("1_incub", "Incubation"),
            ("8_lost", "Abandon"),
            ("9_done", "Accompagnement terminé"),
        ],
        string="Etape d’incubation",
        tracking=True,
        readonly=True,
    )
    project_id = fields.Many2one(
        comodel_name="res.partner",
        string="Projet",
        ondelete="restrict",
        domain=[("contact_type", "=", "project"), ("is_company", "=", True)],
    )
    is_project_contact = fields.Boolean("Référent Projet", default=False, tracking=True)
    incubated_ids = fields.One2many(
        comodel_name="res.partner",
        inverse_name="project_id",
        string="Liste des porteurs de projet",
        domain=[("contact_type", "=", "incubated"), ("is_company", "!=", True)],
    )
    committee_backup = fields.Text("Retours comité")
    # field to set readonly fields if not admin
    is_alterincub_admin = fields.Boolean(compute="_compute_is_alterincub_admin")
    # Contact fields
    birthdate = fields.Date("Date de naissance")
    age_range = fields.Selection(
        selection=[
            ("under_25", "Moins de 25 ans"),
            ("25_35", "Entre 25 et 35 ans"),
            ("36_45", "Entre 36 et 45 ans"),
            ("over_45", "Plus de 45 ans"),
        ],
        string="Tranche d'âge",
        compute="_compute_age_range",
        store=True,
    )
    genre = fields.Selection(
        selection=[
            ("homme", "Homme"),
            ("femme", "Femme"),
        ]
    )
    nationality = fields.Selection(
        selection=[
            ("ue", "UE"),
            ("not_ue", "HORS UE"),
            ("nd", "ND"),
        ],
        string="Nationalité",
    )
    foreign_parents = fields.Selection(
        selection=[
            ("yes", "Oui"),
            ("not", "Non"),
            ("nsp", "NSP"),
        ],
        string="Parents étrangers",
    )
    educational_level = fields.Many2one(
        comodel_name="alterincub.education.level",
        string="Niveau d'instruction",
        ondelete="restrict",
    )
    situation = fields.Many2one(
        comodel_name="alterincub.situation.entrance",
        string="Situation à l’entrée",
        ondelete="restrict",
    )
    disability = fields.Selection(
        selection=[
            ("yes", "Oui"),
            ("no", "Non"),
        ],
        string="Reconnaissance handicap",
    )
    exclusion_risk = fields.Selection(
        selection=[
            ("yes", "Oui"),
            ("not", "Non"),
            ("nsp", "NSP"),
        ],
        string="Risque exclusion",
    )
    prescriber = fields.Selection(
        selection=[
            ("creation_network", "Réseau d’accompagnement à la création"),
            ("cooperative", "SCOP-SCIC"),
            ("financer", "Financeur"),
            ("press", "Article de presse, newsletter"),
            ("website", "Site internet"),
            ("ess_network", "Réseau de l’Economie Sociale et Solidaire"),
            ("event", "Événement"),
            ("other", "Autres"),
        ],
        string="Prescripteur",
    )
    creation_status = fields.Selection(
        selection=[
            ("yes", "Oui"),
            ("no", "Non"),
        ],
        string="Structure créée",
    )
    legal_status = fields.Selection(
        selection=[
            ("association", "Association"),
            ("scic", "Scic"),
            ("scop", "Scop"),
            ("esus_entreprise", "Entreprise agréée ESUS"),
            ("ess_commercial_entreprise", "Entreprise commerciale de l’ESS"),
            ("non_ess_commercial_entreprise", "Entreprise commerciale hors ESS"),
        ],
        string="Statut juridique",
    )
    created_on = fields.Date(
        string="Date de création de la structure",
    )
    still_active = fields.Selection(
        selection=[
            ("yes", "Oui"),
            ("no", "Non"),
        ],
        string="En activité",
    )
    closed_on = fields.Date(
        string="Date de fermeture de la structure",
    )
    sales_range = fields.Selection(
        selection=[
            ("na", "Pas de chiffre d’affaires réalisé"),
            ("under_10k", "Moins de 10 000€"),
            ("10k_25k", "Entre 10 000€ et 25 000€"),
            ("25k_50k", "Entre 25 000€ et 50 000€"),
            ("50k_100k", "Entre 50 000€ et 100 000€"),
            ("100k_200k", "Entre 100 000€ et 200 000€"),
            ("200k_500k", "Entre 200 000€ et 500 000€"),
        ],
        string="Chiffre d'affaires",
    )
    subsidies_range = fields.Selection(
        selection=[
            ("na", "Pas de subventions perçues"),
            ("under_5k", "Moins de 5 000€"),
            ("5k_10k", "Entre 5 000€ et 10 000€"),
            ("10k_25k", "Entre 10 000€ et 25 000€"),
            ("25k_50k", "Entre 25 000€ et 50 000€"),
            ("50k_100k", "Entre 50 000€ et 100 000€"),
            ("100k_200k", "Entre 100 000€ et 200 000€"),
            ("more_200k", "Plus de 200 000€"),
        ],
        string="Montant des subventions",
    )
    employees_range = fields.Selection(
        selection=[
            ("na", "Aucun salarié"),
            ("1_2", "1 à 2 salariés"),
            ("3_5", "3 à 5 salariés"),
            ("6_9", "6 à 9 salariés"),
            ("10_19", "10 à 19 salariés"),
            ("20_49", "20 à 49 salariés"),
            ("more_50", "Plus de 50 salariés"),
        ],
        string="Effectif (nombre de salariés)",
    )
    project_involved = fields.Boolean(
        string="N'est plus actif dans le projet", default=False
    )
    contact_unavailable = fields.Boolean(
        string="Ne souhaite plus être recontacté", default=False
    )
    # ------------------------------------------------------
    # SQL Constraints
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Default methods
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Computed fields / Search Fields
    # ------------------------------------------------------
    @api.depends("type")
    def _compute_parent_id_domain(self):
        """
        Crée un domaine dynamique pour le parent en fonction du menu
        """
        ctx = self._context
        domain = [("is_company", "=", True)]
        for partner in self:
            if ctx.get("default_is_provider", False):
                domain.append(("contact_type", "=", "provider"))
            if ctx.get("default_is_incubated", False):
                domain.append(("contact_type", "=", "project"))
            partner.parent_id_domain = json.dumps(domain)

    @api.depends("parent_id", "parent_id.contact_type")
    def _compute_is_incubated(self):
        for partner in self:
            partner.is_incubated = False
            if partner.parent_id:
                if partner.parent_id.contact_type == "project":
                    partner.is_incubated = True

    @api.depends("parent_id", "parent_id.contact_type")
    def _compute_is_provider(self):
        for partner in self:
            partner.is_provider = False
            if partner.parent_id:
                if partner.parent_id.contact_type == "provider":
                    partner.is_provider = True

    def _compute_is_alterincub_admin(self):
        is_admin = self.env.user.has_group(
            "alterincub_partner.group_alterincub_list_manager"
        )
        for partner in self:
            partner.is_alterincub_admin = False
            if is_admin:
                partner.is_alterincub_admin = True

    @api.depends("name", "contact_type")
    def _compute_original_name(self):
        for partner in self:
            if partner.contact_type == "project" and not partner.original_name:
                partner.original_name = partner.name

    @api.depends("birthdate")
    def _compute_age_range(self):
        for partner in self:
            if partner.birthdate:
                age = relativedelta(fields.Date.today(), partner.birthdate).years
                if age < 25:
                    partner.age_range = "under_25"
                elif 25 <= age <= 35:
                    partner.age_range = "25_35"
                elif 36 <= age <= 45:
                    partner.age_range = "36_45"
                else:
                    partner.age_range = "over_45"
            else:
                partner.age_range = None

    # ------------------------------------------------------
    # Onchange / Constraints
    # ------------------------------------------------------
    @api.onchange("is_company")
    def onchange_is_company(self):
        """
        Set contact_type to False for person
        """
        if not self.is_company:
            self.contact_type = False
            if self.is_incubated or self.is_provider:
                self.type = "other"

    @api.onchange("parent_id")
    def onchange_parent_id(self):
        """
        Override parent method to add default_parent_id
        """
        result = super().onchange_parent_id()
        if self.env.context.get("default_parent_id"):
            self.parent_id = self.env.context.get("default_parent_id")
        if not self.parent_id:
            self.type = "contact"
        else:
            self.type = "other"
        return result

    # ------------------------------------------------------
    # CRUD methods (ORM overrides)
    # ------------------------------------------------------
    def write(self, vals):
        if vals.get("project_state") and vals.get("project_state") == "8_lost":
            if not vals.get("end_reason_id"):
                raise UserError(
                    _(
                        "Un projet ne peut être abandonné sans motif.\n"
                        "Veuillez utiliser le bouton approprié pour basculer le projet "
                        "au statut Abandonné."
                    )
                )
        if vals.get("project_state") and vals.get("project_state") != "8_lost":
            for partner in self:
                if partner.end_reason_id:
                    raise UserError(
                        _(
                            "Le projet %s a été abandonné, il n'est plus possible de "
                            "le changer de statut."
                        )
                        % (partner.name,)
                    )

        return super().write(vals)

    # ------------------------------------------------------
    # Actions
    # ------------------------------------------------------
    def update_price_list(self):
        for partner in self:
            if partner.lot_ids:
                price_obj = self.env["alterincub.price.list"]
                service_place_ids = self.env["alterincub.service.place"].search([])
                type_lot = partner.lot_ids.mapped("type_lot")
                for tl in type_lot:
                    # ruff: noqa: B023
                    sp_ids = service_place_ids.filtered(lambda p: p.type_lot == tl)
                    for place in sp_ids:
                        price_exists = price_obj.search(
                            [
                                ("partner_id", "=", partner.id),
                                ("service_place_id", "=", place.id),
                            ]
                        )
                        if not price_exists:
                            price_obj.create(
                                {
                                    "partner_id": partner.id,
                                    "service_place_id": place.id,
                                    "type_lot": tl,
                                }
                            )

    # ------------------------------------------------------
    # CRON methods
    # ------------------------------------------------------
    def cron_age_range(self):
        """
        CRON permettant de recalculer les tranches d'âge
        """
        partner_ids = self.search([("birthdate", "!=", False)])
        partner_ids._compute_age_range()

    # ------------------------------------------------------
    # Business methods
    # ------------------------------------------------------
    def get_inline_address(self):
        self.ensure_one()
        address_fields = [self.street, self.street2, self.zip, self.city]
        address = [f for f in address_fields if f]
        return " ".join(address)

    def get_formatted_address(self):
        self.ensure_one()
        address_fields = [self.street, self.street2]
        address = [f for f in address_fields if f]
        return "\n".join(address) if address else ""

    def set_project_to_incub(self):
        self.ensure_one()
        self.project_state = "1_incub"
